@@ -144,9 +144,9 @@ static GnomeUIInfo menu_defaults[] = {
           GNOME_APP_PIXMAP_STOCK, GTK_STOCK_CLOSE,
           GNOME_KEY_NAME_CLOSE, GNOME_KEY_MOD_CLOSE, NULL },
 	/* Exit */
-        { GNOME_APP_UI_ITEM, N_("E_xit"), N_("Exit the program"),
+        { GNOME_APP_UI_ITEM, N_("_Quit"), N_("Quit the application"),
           NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-	  GTK_STOCK_QUIT, GNOME_KEY_NAME_EXIT, GNOME_KEY_MOD_EXIT,
+	  GTK_STOCK_QUIT, GNOME_KEY_NAME_QUIT, GNOME_KEY_MOD_QUIT,
 	    NULL },
 	/*
 	 * The "Edit" menu
@@ -299,7 +299,7 @@ static gchar *menu_names[] =
   "print",
   "print-setup",
   "close",
-  "exit",
+  "quit",
   "cut",
   /* 10 */
   "copy",
@@ -1236,6 +1236,51 @@ gnome_app_fill_menu_with_data (GtkMenuShell  *menu_shell,
 				    pos);
 }
 
+static void
+menus_have_tearoff_changed_notify(GConfClient            *client,
+				  guint                   cnxn_id,
+				  GConfEntry             *entry,
+				  gpointer                user_data)
+{
+	GtkWidget *menu;
+	
+	if (entry->value->type != GCONF_VALUE_BOOL)
+		return;
+
+	menu = GTK_WIDGET (user_data);
+	
+	if (gconf_value_get_bool (entry->value)) {
+		GtkWidget *tearoff;
+
+		tearoff = g_object_get_data (G_OBJECT (menu), "gnome-app-tearoff");
+
+		if (tearoff) {
+			/* Do nothing */
+			return;
+		}
+		
+		/* Add the tearoff */
+		tearoff = gtk_tearoff_menu_item_new ();
+		gtk_widget_show (tearoff);
+		g_object_set_data (G_OBJECT (menu), "gnome-app-tearoff", tearoff);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), tearoff);
+	}
+	else {
+		GtkWidget *tearoff;
+
+		tearoff = g_object_get_data (G_OBJECT (user_data), "gnome-app-tearoff");
+
+		if (!tearoff) {
+			/* Do nothing */
+			return;
+		}
+		
+		/* Remove the tearoff */
+		gtk_widget_destroy (tearoff);
+		g_object_set_data (G_OBJECT (menu), "gnome-app-tearoff", NULL);
+	}
+}
+
 
 /**
  * gnome_app_fill_menu_custom
@@ -1326,7 +1371,9 @@ gnome_app_fill_menu_custom (GtkMenuShell       *menu_shell,
 
 				GtkWidget *menu;
 				GtkWidget *tearoff;
-
+				guint notify_id;
+				GConfClient *client;
+				
 				menu = gtk_menu_new ();
 				gtk_menu_item_set_submenu
 					(GTK_MENU_ITEM(uiinfo->widget), menu);
@@ -1336,12 +1383,25 @@ gnome_app_fill_menu_custom (GtkMenuShell       *menu_shell,
 					 uiinfo->moreinfo, orig_uibdata,
 					 accel_group, uline_accels, 0);
 
-				/* FIXME: make this runtime configurable */
 				if (gnome_gconf_get_bool ("/desktop/gnome/interface/menus_have_tearoff")) {
 					tearoff = gtk_tearoff_menu_item_new ();
 					gtk_widget_show (tearoff);
+					g_object_set_data (G_OBJECT (menu), "gnome-app-tearoff", tearoff);
 					gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), tearoff);
 				}
+
+				client = gconf_client_get_default ();
+				g_object_set_data_full(G_OBJECT(menu), gnome_app_helper_gconf_client,
+						       client, g_object_unref);
+
+				notify_id = gconf_client_notify_add (client,
+								     "/desktop/gnome/interface/menus_have_tearoff",
+								     menus_have_tearoff_changed_notify,
+								     menu, NULL, NULL);
+				g_signal_connect(menu, "destroy",
+						 G_CALLBACK(remove_notify_cb),
+						 GINT_TO_POINTER(notify_id));
+
 			}
 			pos++;
 			break;
@@ -2260,7 +2320,8 @@ gnome_app_helper_gettext (const gchar *str)
 static GConfEnumStringPair toolbar_styles[] = {
         { GTK_TOOLBAR_TEXT, "text" },
         { GTK_TOOLBAR_ICONS, "icons" },
-        { GTK_TOOLBAR_BOTH, "both" }
+        { GTK_TOOLBAR_BOTH, "both" },
+        { GTK_TOOLBAR_BOTH_HORIZ, "both_horiz" }
 };
 
 static void
@@ -2388,9 +2449,9 @@ static void
 create_and_popup_toolbar_menu (GdkEventButton *event)
 {
 	GtkWidget *menu;
-	GtkWidget *item, *both_item, *icons_item, *text_item, *global_item;
+	GtkWidget *item, *both_item, *both_horiz_item, *icons_item, *text_item, *global_item;
 	GSList *group;
-	char *both, *icons, *text, *global;
+	char *both, *both_horiz, *icons, *text, *global;
 	char *str, *key;
 	GtkToolbarStyle toolbar_style;
 	GConfClient *conf;
@@ -2400,6 +2461,7 @@ create_and_popup_toolbar_menu (GdkEventButton *event)
 	menu = gtk_menu_new ();
 
 	both = _("Both text and icons");
+	both_horiz = _("Both text and icons when horizontal, just icons when vertical");
 	icons = _("Icons only");
 	text = _("Text only");
 
@@ -2409,6 +2471,12 @@ create_and_popup_toolbar_menu (GdkEventButton *event)
 	group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (both_item));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), both_item);
 
+	both_horiz_item = gtk_radio_menu_item_new_with_label (group, both_horiz);
+	g_signal_connect (both_item, "activate",
+			  G_CALLBACK (style_menu_item_activated), GINT_TO_POINTER (GTK_TOOLBAR_BOTH_HORIZ));
+	group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (both_horiz_item));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), both_horiz_item);
+
 	icons_item = gtk_radio_menu_item_new_with_label (group, icons);
 	g_signal_connect (icons_item, "activate",
 			  G_CALLBACK (style_menu_item_activated), GINT_TO_POINTER (GTK_TOOLBAR_ICONS));
@@ -2442,6 +2510,9 @@ create_and_popup_toolbar_menu (GdkEventButton *event)
 	case GTK_TOOLBAR_BOTH:
 		str = both;
 		break;
+	case GTK_TOOLBAR_BOTH_HORIZ:
+		str = both_horiz;
+		break;
 	case GTK_TOOLBAR_ICONS:
 		str = icons;
 		break;
@@ -2481,6 +2552,9 @@ create_and_popup_toolbar_menu (GdkEventButton *event)
 		case GTK_TOOLBAR_BOTH:
 			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (both_item), TRUE);
 			break;
+		case GTK_TOOLBAR_BOTH_HORIZ:
+			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (both_horiz_item), TRUE);
+			break;
 		case GTK_TOOLBAR_ICONS:
 			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (icons_item), TRUE);
 			break;
